package me.unurled.srcore;

import static me.unurled.srcore.utils.LoggerUtils.error;
import static me.unurled.srcore.utils.ReflectionRegistry.*;

import me.unurled.srcore.utils.ReflectionUtils;
import org.bukkit.plugin.Plugin;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/** The SRCore class is a class that is used to manage the core of the plugin. */
public final class SRCore {

  private static SRCore instance;
  private final Managers managers;
  private Plugin plugin;

  private SRCore() {
    managers = new Managers();
  }

  /**
   * Get an instance of the SRCore
   *
   * @return The instance of the SRCore
   */
  public static @NotNull SRCore getInstance() {
    return instance == null ? new SRCore() : instance;
  }

  /**
   * Get the plugin which called the api
   *
   * @return The plugin
   */
  public @NotNull Plugin getPlugin() {
    if (plugin == null) {
      setPlugin(tryFindPlugin());

      if (plugin == null)
        throw new IllegalStateException(
            "Plugin is not set. Set it using InvUI.getInstance().setPlugin(plugin);");
    }

    return plugin;
  }

  /**
   * Set the plugin which called the api
   *
   * @param plugin The plugin
   */
  public void setPlugin(@Nullable Plugin plugin) {
    if (this.plugin != null) throw new IllegalStateException("Plugin is already set");

    if (plugin == null) return;

    this.plugin = plugin;
  }

  private @Nullable Plugin tryFindPlugin() {
    ClassLoader loader = getClass().getClassLoader();

    try {
      if (PLUGIN_CLASS_LOADER_CLASS.isInstance(loader)) {
        return ReflectionUtils.getFieldValue(PLUGIN_CLASS_LOADER_PLUGIN_FIELD, loader);
      } else if (PAPER_PLUGIN_CLASS_LOADER_CLASS != null
          && PAPER_PLUGIN_CLASS_LOADER_CLASS.isInstance(loader)) {
        return ReflectionUtils.invokeMethod(
            PAPER_PLUGIN_CLASS_LOADER_GET_LOADED_JAVA_PLUGIN_METHOD, loader);
      }
    } catch (Exception e) {
      error(e.getMessage());
    }

    return null;
  }

  /**
   * Get the managers for every manager
   *
   * @return The managers instance
   */
  public Managers getManagers() {
    return managers;
  }

  /** Unload the managers, to be called when the plugin is disabled */
  public void unload() {
    managers.unload();
  }
}
