package me.unurled.srcore.utils;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/** The ReflectionUtils class is a class that is used to reflect classes and methods. */
public class ReflectionUtils {

  private ReflectionUtils() {}

  /**
   * Get a class from a path
   *
   * @param path The path of the class
   * @return The class
   * @param <T> a type of class
   */
  public static <T> @NotNull Class<T> getClass(@NotNull String path) {
    try {
      return (Class<T>) Class.forName(path);
    } catch (Exception e) {
      throw new RuntimeException(e.getMessage());
    }
  }

  /**
   * Get a class from a path
   *
   * @param path The path of the class
   * @return The class or null
   * @param <T> a type of class
   */
  public static <T> @Nullable Class<T> getClassOrNull(@NotNull String path) {
    try {
      return (Class<T>) Class.forName(path);
    } catch (Exception e) {
      return null;
    }
  }

  /**
   * Get a method from a class
   *
   * @param clazz The class
   * @param declared If the method is declared
   * @param name The name of the method
   * @param parameterTypes The parameter types of the method
   * @return The method
   */
  public static @Nullable Method getMethodOrNull(
      @NotNull Class<?> clazz,
      boolean declared,
      @NotNull String name,
      @NotNull Class<?> @NotNull ... parameterTypes) {
    try {
      Method method =
          declared
              ? clazz.getDeclaredMethod(name, parameterTypes)
              : clazz.getMethod(name, parameterTypes);
      if (declared) method.setAccessible(true);
      return method;
    } catch (Exception e) {
      return null;
    }
  }

  /**
   * Get a method from a class
   *
   * @param clazz The class
   * @param declared If the method is declared
   * @param name The name of the method
   * @return The method
   */
  public static @NotNull Field getField(
      @NotNull Class<?> clazz, boolean declared, @NotNull String name) {
    try {
      Field field = declared ? clazz.getDeclaredField(name) : clazz.getField(name);
      if (declared) field.setAccessible(true);
      return field;
    } catch (Exception e) {
      throw new RuntimeException(e.getMessage());
    }
  }

  /**
   * Get a field from a class
   *
   * @param field The field
   * @param obj The object
   * @return The field value
   * @param <T> a type of field
   */
  public static <T> @Nullable T getFieldValue(@NotNull Field field, @Nullable Object obj) {
    try {
      return (T) field.get(obj);
    } catch (Exception e) {
      throw new RuntimeException(e.getMessage());
    }
  }

  /**
   * Invoke a method
   *
   * @param method The method
   * @param obj The object
   * @param args The arguments
   * @return The method value
   * @param <T> a type of method
   */
  public static <T> T invokeMethod(
      @NotNull Method method, @Nullable Object obj, @Nullable Object @Nullable ... args) {
    try {
      return (T) method.invoke(obj, args);
    } catch (Exception e) {
      throw new RuntimeException(e.getMessage());
    }
  }
}
